using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.ComponentModel.DataAnnotations.Schema;
using System.Linq;

namespace App.Common
{
    public class CourseEnrolment : Entity
    {
        //Hugh Nichols
        [Required]
        public virtual TrainingOrganisationDeliveryLocation TrainingOrganisationDeliveryLocation { get; set; }
        public virtual long TrainingOrganisationDeliveryLocationId { get; set; }
        [Required]
        public virtual Course Course { get; set; }
        public virtual long CourseId { get; set; }
        [Required]
        public virtual Enrolment Enrolment { get; set; }
        public virtual long EnrolmentId { get; set; }
        [Required]
        public virtual Student Student { get; set; }
        public virtual long StudentId { get; set; }
        [Required]
        public virtual CourseIntake CourseIntake { get; set; }
        public virtual long CourseIntakeId { get; set; }

        public DateTime? DateEnrolled { get; set; } // UNUSED!!
        public DateTime? DatePassedTheory { get; set; }
        public DateTime? DateCompleted { get; set; }
        public DateTime? DateWithdrawn { get; set; }

        public virtual ICollection<Doc> Docs { get; set; }

        [NotMapped]
        public string ProgressDisp
        {
            get
            {
                if (Course == null) return "";
                if (Student == null) return "";
                if (lessonPlans == null) lessonPlans = LessonPlans;
                return String.Format("{0}/{1}", lessonPlans.Count(x => x.SkillsFinished), lessonPlans.Count);
            }
        }

        [NotMapped]
        public bool HasCompleted { get { return DateCompleted != null; } }
        [NotMapped]
        public bool HasWithdrawn { get { return DateWithdrawn != null; } }
        [NotMapped]
        public bool IsCurrent { get { return !HasCompleted && !HasWithdrawn; } }
        [NotMapped]
        public bool HasPassedTheory { get { return DatePassedTheory != null; } }

        [NotMapped]
        public string EnrolmentStatusDisp { get { return IsCurrent ? "Current" : (HasCompleted ? "Completed" : "Withdrawn"); } }

        [NotMapped]
        public string CourseName { get { return Course == null ? "" : Course.Name; } }

        private ICollection<CourseUnit> lessonPlans;
        [NotMapped]
        public virtual ICollection<CourseUnit> LessonPlans
        {
            get
            {
                if (Course == null) return null;
                if (Student == null) return null;
                lessonPlans = Course.CourseUnits.Where(x => !x.Deleted).ToArray();
                foreach (var unit in lessonPlans)
                {
                    var skillsDone = Student.Lessons.SelectMany(x => x.LessonSkills).Where(x => x.CourseSkill.CourseUnitId == unit.Id && x.Status == LessonSkillStatus.Completed).Select(x => x.CourseSkillId).ToArray();
                    unit.SkillsDoneQty = skillsDone.Count();
                    var skillsNotDoneQty = Student.Lessons.SelectMany(x => x.LessonSkills).Count(x => x.CourseSkill.CourseUnitId == unit.Id && !skillsDone.Contains(x.CourseSkillId));
                    if (unit.SkillsDoneQty > 0 && unit.SkillsQty > 0 && unit.SkillsDoneQty % unit.SkillsQty == 0 && skillsNotDoneQty > 0)
                    {
                        unit.SkillsDoneQty = 0;
                        unit.SkillsDoneRepeating = true;
                    }
                    while (unit.SkillsDoneQty > unit.SkillsQty) unit.SkillsDoneQty -= unit.SkillsQty;
                    unit.TotalHrs = Student.Lessons.SelectMany(x => x.LessonUnits).Where(x => x.CourseUnitId == unit.Id).Sum(x => x.TotalHrs);
                }
                return lessonPlans;
            }
        }
        [NotMapped]
        public virtual ICollection<CourseUnit> LessonPlansOutstanding
        {
            get { if (LessonPlans == null) return null; return LessonPlans.Where(x => !x.SkillsFinished).ToList(); }
        }

        public string StudyReasonId { get; set; }
        public string FeeExemptionConcessionTypeId { get; set; }
        public bool IssuedFlag { get; set; }

        [Required]
        public virtual PaymentType PaymentType { get; set; }
        public virtual long PaymentTypeId { get; set; }

        public bool RplRequested { get; set; }

        public decimal LoanLimit { get; set; }
        public decimal LoanFee { get; set; }
        public bool FeesApportioned { get; set; }

        public virtual ICollection<CourseFee> CourseFees { get; set; }

        [NotMapped]
        public string LoanProvider => PaymentType?.LoanProvider ?? "";
        [NotMapped]
        public bool IsLoan => !string.IsNullOrEmpty(LoanProvider);
        [NotMapped]
        public bool CanMatchXero => !FeesApportioned;
        [NotMapped]
        public bool CanApportionFees => !FeesApportioned && Student?.XeroContact != null;
    }
}
