﻿using System;
using System.Collections.Generic;
using System.Linq;
using App.Common;
using App.Services;
using Wisej.Web;

namespace App.UI.Pages.Training
{
    public partial class CourseFees : Wisej.Web.UserControl
    {
        private ApplicationService _service;

        public CourseFees()
        {
            InitializeComponent();
            //this.groupBox1.DataBindings.Add(new Wisej.Web.Binding("Enabled", this.bsEnrolment, "CanApportionFees", true));
            //this.groupBox3.DataBindings.Add(new Wisej.Web.Binding("Enabled", this.bsEnrolment, "CanMatchXero", true));
        }

        private void TrainingFees_Load(object sender, EventArgs e)
        {
            this.Enabled = false;
        }

        public void LoadFees(ApplicationService service, CourseEnrolment enrolment)
        {
            if (enrolment == null) { this.Enabled = false; return; }
            _service = service;
            comboBoxPaymentType.DataSource = service.GetPicklist<PaymentType>();
            comboBoxPaidBy.Items.Clear();
            comboBoxPaidBy.Items.Add("Student");
            if (!string.IsNullOrEmpty(enrolment.LoanProvider)) comboBoxPaidBy.Items.Add("External Loan Provider");
            dtAdjustDueDate.Value = DateTime.Today;
            ShowFees(enrolment);
        }

        private void ShowFees(CourseEnrolment enrolment)
        {
            if (enrolment == null) { this.Enabled = false; return; }
            bsEnrolment.DataSource = enrolment;
            bsEnrolment.ResetBindings(false);
            bsCourseFee.DataSource = null;
            bsCourseFee.DataSource = enrolment.CourseFees;
            this.Enabled = true;
            //Adjustment
            comboBoxPaidBy.Enabled = enrolment.FeesApportioned;
            comboBoxPaidBy.SelectedIndex = enrolment.FeesApportioned ? Math.Max(comboBoxPaidBy.SelectedIndex, 0) : -1;
            if (enrolment.FeesApportioned) comboBoxAdjustPeriod.DataSource = _service.GetInvoicePeriods(enrolment);
            else comboBoxAdjustPeriod.SelectedIndex = -1;
            comboBoxAdjustPeriod.Enabled = enrolment.FeesApportioned;
            //Xero
            bsXeroContacts.DataSource = _service.GetXeroContactMatches(enrolment.Student);
            listBoxXero.SelectedIndex = -1;
            listBoxXero.SelectedIndex = 0;
            //Apportion
            textBoxLoanFeePercent.Text = "";
            textBoxInitialInvoice.Text = "";
            panelInvoiceDates.Controls.Clear();
            if (enrolment.CanApportionFees) ShowApportionFees(enrolment);
        }

        private void ShowApportionFees(CourseEnrolment enrolment)
        {
            textBoxLoanFeePercent.Text = enrolment.PaymentType.FeePercent.ToString();
            var (initialPayment, offsets) = _service.ShowApportionFees(enrolment);
            textBoxInitialInvoice.Text = initialPayment;
            foreach (var offset in offsets)
            {
                var picker = new DateTimePicker { CustomFormat = "dd/MM/yyyy", Dock = DockStyle.Top, Format = DateTimePickerFormat.Custom, Margin = new Padding(3, 3, 3, 10), Text = "", Tag = offset };
                panelInvoiceDates.Controls.Add(picker);
                if (offset == 0) picker.ValueChanged += FirstInvoiceDate_ValueChanged;
            }
        }

        private void FirstInvoiceDate_ValueChanged(object sender, EventArgs e)
        {
            var first = (DateTimePicker)sender;
            foreach (Control control in panelInvoiceDates.Controls)
            {
                var dt = (DateTimePicker)control;
                var offset = int.Parse(dt.Tag?.ToString() ?? "0");
                if (offset > 0) dt.Value = first.Value.AddDays(offset);
            }
        }

        private void buttonApportion_Click(object sender, EventArgs e)
        {
            var enrolment = (CourseEnrolment)bsEnrolment.Current;

            if (enrolment.IsLoan && enrolment.LoanLimit == 0)
                if (MessageBox.Show("There is a fee help provider specified with a limit of $0.  No fees will be invoiced to the fee help provider.  ARE YOU SURE?", "Confirm", MessageBoxButtons.YesNo, MessageBoxIcon.Warning) != DialogResult.Yes) return;
            var dates = new List<DateTimePicker>(); foreach (Control control in panelInvoiceDates.Controls) dates.Add((DateTimePicker)control);
            decimal.TryParse(textBoxInitialInvoice.Text, out var initialInvoice);
            var invoiceDates = dates.OrderBy(x => int.Parse(x.Tag?.ToString() ?? "0")).Select(x => x.Value).ToArray();
            _service.ApportionCourseFees(enrolment, initialInvoice, invoiceDates);
            ShowFees(enrolment);
        }

        private void comboBoxAdjustPeriod_SelectedIndexChanged(object sender, EventArgs e)
        {
            dtAdjustDueDate.Value = comboBoxAdjustPeriod.SelectedIndex == -1 || !(comboBoxAdjustPeriod.SelectedItem is StudyPeriod) ? DateTime.Today : ((StudyPeriod)comboBoxAdjustPeriod.SelectedItem)?.CensusDate ?? DateTime.Today;
        }

        private void buttonAddAdjustment_Click(object sender, EventArgs e)
        {
            var enrolment = (CourseEnrolment)bsEnrolment.Current;
            var description = textBoxAdjustDescription.Text;
            decimal.TryParse(textBoxAdjustAmount.Text, out var amount);
            var loan = comboBoxPaidBy.SelectedIndex == 1;
            var period = comboBoxAdjustPeriod.Text;
            var due = dtAdjustDueDate.Value;
            _service.AddCourseFeeAdjustment(enrolment, description, amount, loan, period, due);
            textBoxAdjustDescription.Text = "";
            textBoxAdjustAmount.Text = "";
            ShowFees(enrolment);
        }

        private void buttonCalculateLoanFee_Click(object sender, EventArgs e)
        {
            var enrolment = (CourseEnrolment)bsEnrolment.Current;
            decimal.TryParse(textBoxLoanFeePercent.Text, out var percent);
            enrolment.LoanFee = _service.CalculateLoanFee(enrolment, percent);
            bsEnrolment.ResetBindings(false);
        }

        private void buttonRefreshXeroContacts_Click(object sender, EventArgs e)
        {
            _service.UpdateFromXero();
            var enrolment = (CourseEnrolment)bsEnrolment.Current;
            ShowFees(enrolment);
        }

        private void buttonXeroContactAction_Click(object sender, EventArgs e)
        {
            var enrolment = (CourseEnrolment)bsEnrolment.Current;
            var xeroContact = (XeroContact)listBoxXero.SelectedItem;
            _service.XeroContactAction(buttonXeroContactAction.Text, enrolment.Student, xeroContact);
            ShowFees(enrolment);
        }

        private void listBoxXero_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (listBoxXero.SelectedIndex < 0) return;
            var student = ((CourseEnrolment)bsEnrolment.Current).Student;
            var xeroContact = (XeroContact)listBoxXero.SelectedItem;
            buttonXeroContactAction.Text = xeroContact.Id == student.XeroContactId ? "Unmatch" : (xeroContact.Id == 0 ? "Create" : "Match");
        }

        private void comboBoxPaymentType_LostFocus(object sender, EventArgs e)
        {
            _service.SaveChanges();
            var enrolment = (CourseEnrolment)bsEnrolment.Current;
            ShowFees(enrolment);
        }
    }
}
